unit pcidio8255Unit1;
{$R+}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ScktComp, ActnList, Menus, ExtCtrls, An8255Frame, An8255CardFrame;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    CardsPanel: TPanel;
    Splitter1: TSplitter;
    CardListPanel: TPanel;
    CardList: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Splitter1Moved(Sender: TObject);
    procedure FormCanResize(Sender: TObject; var NewWidth, NewHeight: Integer; var Resize: Boolean);
  private
    bSampleInitialized: Boolean;
    function FindCards(con: TStrings):Boolean;
    procedure doDisplay(I, G: Integer; DataToDisplay: TDIO8255Data);
    function ShowFrame(FrameName: string; C, G, X, Y:integer): TFrame;

    procedure PerformIdle(Sender: TObject; var Done: Boolean);
    { Private declarations }
  public
    { Public declarations }
    CardInfo : array of TOne8255CardFrame;
  end; // TForm1

var
  Form1: TForm1;
const
  MARGIN8255 = 4;
  CARDMARGIN = 4;

implementation
  uses AIOWDM;

{$R *.DFM}
var
  OriginalCardListPanelHeight: integer;
  OriginalHeight: Integer;

function TForm1.FindCards(con:TStrings):Boolean;
  const
    bmDIO8255 = $0EFF;                                // Pure DIO boards can only have these bits set
    bmDIOType = $00C0;                                // These bits set what type of DIO is on the card
    bDIOTypeIs8255 = $0040;                           // the DIOType bits must be this value to be 8255-style DIO
    bmNumChannels = $0031;                            // These 3 bits enumerate how many DIO bits are on the card
    bmIsolated = $0100;                               // This bit is set on Isolated devices
    NumChannelsToNumGroups: array[0..7] of integer = (0, 1, 1, 2, 2, 3, 4, 5);
  Var
    NumberOfAIOWDMCardsFound: Integer;
    C, N, Y, G, DIOCardCount, DeviceID: Integer;
    Note: String;
begin
  Result := True;
  Y := 4;
  G := 0;
  DIOCardCount := 0;

  SetLength( Cardinfo, 1 );

  for C := 0 to high(CardInfo) do
  begin
    DeviceID := $0C60;
    if (DeviceID and (not bmDIO8255)) <> 0 then
    begin
      Note := 'non-DIO board, Not for this Sample';
    end else
      if (DeviceID and bmDIOType) = bDIOTypeIs8255 then
      begin
        if (DeviceID and bmIsolated) = bmIsolated then
        begin
          Note := 'Isolated Digital Card, not for this Sample';
        end else begin
          Result := True;
          CardInfo[DIOCardCount] := ShowFrame('One8255CardFrame', DIOCardCount, G, 0, Y) as TOne8255CardFrame;
          CardInfo[DIOCardCount].Num8255 := NumChannelsToNumGroups[ (DeviceID and bmNumChannels) shr 3 ];

          N := MAX_PATH;
          SetLength( CardInfo[DIOCardCount].Name, N );
          QueryCardInfo( DIOCardCount, @CardInfo[DIOCardCount].DeviceID, nil, @N, PChar(CardInfo[DIOCardCount].Name) );
          SetLength( CardInfo[DIOCardCount].Name, N -1 );  // delete trailing null

          case (CardInfo[DIOCardCount].Num8255) of
            1: begin Note := '24 DIO bits (1 50-Pin connector)';  end;
            2: begin Note := '48 DIO bits (2 50-Pin connectors)'; end;
            3: begin Note := '72 DIO bits (3 50-Pin connectors)'; end;
            4: begin Note := '96 DIO bits (4 50-Pin connectors)'; end;
            5: begin Note := '120 DIO bits (5 50-Pin connectors)'; end;
          else
            begin
              Note := 'Unknown DIO configuration, Might not be for this sample';
            end
          end;// switch
          CardInfo[DIOCardCount].Note := Note;
          Note :=  CardInfo[DIOCardCount].Name + ' - ' + Note;
          inc(DIOCardCount);
        end;// non-isolated DIO (therefore an 8255) board
      end;
    con.Add( IntToHex(C, 2) + ': ' + IntToHex(DeviceID, 4) + ' - ' + Note );
  end;
  con.Add( IntToStr(NumberOfAIOWDMCardsFound) + ' card(s) found' );
  SetLength( Cardinfo, DIOCardCount );
end; // end FindCards

procedure TForm1.FormCreate(Sender: TObject);
begin
  bSampleInitialized := False;
  if FindCards( CardList.Lines ) then
    Application.OnIdle := Form1.PerformIdle;
end;

procedure TForm1.PerformIdle(Sender: TObject; var Done: Boolean);
  var
    C, X: Integer;
begin
  Application.OnIdle := nil;
  X := 4;
  for C := 0 to high(CardInfo) do
  begin
    if CardInfo[C].BuildCardFrames(C, X, 0) then
    begin
      X := X + CardInfo[C].Width + CARDMARGIN; // 4 is the between-cards margin
    end;
  end;

  Form1.ClientWidth := CardInfo[high(CardInfo)].Width + Cardinfo[high(CardInfo)].Left + CARDMARGIN + (high(cardinfo)-1) * CARDMARGIN;
  Form1.ClientHeight := CardInfo[high(CardInfo)].Height + CARDMARGIN*2 + CardListPanel.Height;
  OriginalHeight := Height;
  OriginalCardListPanelHeight := CardListPanel.height;
  bSampleInitialized := True;

end;

procedure TForm1.Timer1Timer(Sender: TObject);
  var
    C, G: integer;
begin
  Timer1.Enabled := False;
  for C := 0 to High(CardInfo) do
    for G := 0 to High(CardInfo[C].Group) do
    begin
      CardInfo[C].Group[G].doUpdate;
      //doDisplay(C, G, CardInfo[C].Group[G].An8255.DIO);
    end;
  Timer1.Enabled := True;
end;

procedure TForm1.doDisplay(I, G:Integer; DataToDisplay: TDIO8255Data);
begin
  CardList.Lines.add( 'card:' + IntToStr(I) + ' group:' + IntToStr(G) + ' data:' + IntToHex(DataToDisplay.Data,6) );
end;

type
  TFrameClass = class of TFrame;

function TForm1.ShowFrame(FrameName: string; C, G, X, Y:integer): TFrame;
var
  Frame: TFrame;
  FrameClass: TFrameClass;
begin
  Frame := FindComponent (FrameName+ IntToHex(C, 2) + IntToHex(G, 4)) as TFrame;
  if not Assigned (Frame) then
  begin
    FrameClass := TFrameClass (FindClass ('T' + FrameName));
    Frame := FrameClass.Create (Form1);
    Frame.Parent := CardsPanel;
    Frame.Visible := True;
    Frame.Left := X;
    Frame.Top := Y;
    Frame.Tag := G;
    Frame.Name := FrameName+ IntToHex(C, 2) + IntToHex(G, 4);
  end;
  Result := Frame;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  CardInfo[0].AutoSize := True;
end;

procedure TForm1.Splitter1Moved(Sender: TObject);
begin
  //Caption := IntToStr(splitter1.top);
  height := OriginalHeight + (CardListPanel.height - OriginalCardListPanelHeight);
end;

procedure TForm1.FormCanResize(Sender: TObject; var NewWidth, NewHeight: Integer; var Resize: Boolean);
begin
//  if bSampleInitialized = True then
//    CardListPanel.height := OriginalCardListPanelHeight + (newheight-height);
end;

end.
